using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;

using ArtOfTest.WebAii.Controls.HtmlControls;
using ArtOfTest.WebAii.Controls.HtmlControls.HtmlAsserts;
using ArtOfTest.WebAii.Core;
using ArtOfTest.WebAii.ObjectModel;
using ArtOfTest.WebAii.TestAttributes;
using ArtOfTest.WebAii.TestTemplates;
using ArtOfTest.WebAii.Win32.Dialogs;

using ArtOfTest.WebAii.Silverlight;
using ArtOfTest.WebAii.Silverlight.UI;

using NUnit.Framework;
using Core = NUnit.Core;


using Telerik.WebAii.Controls.Html;
using TelerikWebApp2;

namespace TelerikWebApp2.Tests
{
    [TestFixture]
    public class TestBasicSetupRecipe : WebTestBase
    {
        #region ClassAttributes
        static List<GUIBasicRecipeLocationMessage> s_locationMessages = new List<GUIBasicRecipeLocationMessage>();


        protected const string SAVE_BUTTON = "ctl00_ContentPlaceHolder1_BasicSRUC_SaveButton";
        protected const string RESET_BUTTON = "ctl00_ContentPlaceHolder1_BasicSRUC_ResetButton";

        protected const string BUMP_LOCATION_GRID = "ctl00_ContentPlaceHolder1_BasicSRUC_BumpLocationGrid";
        protected const int BUMP_LOCATION_GUI_INDEX_CELL = 0;
        protected const int BUMP_LOCATION_XPOSITION_CELL = 1;
        protected const int BUMP_LOCATION_YPOSITION_CELL = 2;

        protected const float SEED_INCREMENT = 0.5F;
        protected const float SEED_START = 0.1F;

        #endregion ClassAttributes


        [SetUp]
        public override void Setup()
        {
            base.Setup();
            if (s_locationMessages.Count != 0) s_locationMessages.Clear();

            s_locationMessages.Add(new GUIBasicRecipeLocationMessage { GUIIndex = 0, x_coord = 0.0F, y_coord = 0.0F, bz = 0.0F, phi = 0.0F, theta = 0.0F });
            s_locationMessages.Add(new GUIBasicRecipeLocationMessage { GUIIndex = 1, x_coord = 1.0F, y_coord = 1.0F, bz = 0.0F, phi = 0.0F, theta = 0.0F });
            s_locationMessages.Add(new GUIBasicRecipeLocationMessage { GUIIndex = 2, x_coord = 2.0F, y_coord = 2.0F, bz = 0.0F, phi = 0.0F, theta = 0.0F });
            s_locationMessages.Add(new GUIBasicRecipeLocationMessage { GUIIndex = 3, x_coord = 3.0F, y_coord = 3.0F, bz = 0.0F, phi = 0.0F, theta = 0.0F });
            s_locationMessages.Add(new GUIBasicRecipeLocationMessage { GUIIndex = 4, x_coord = 4.0F, y_coord = 4.0F, bz = 0.0F, phi = 0.0F, theta = 0.0F });
        }

        [Test]
        public void TestLoadPage()
        {
            ActiveBrowser.NavigateTo(BASIC_SETUP_RECIPE_PAGE);
            Assert.AreEqual(BASIC_SETUP_RECIPE_PAGE_TITLE, ActiveBrowser.PageTitle, "Actual page title = \"{0}\"", ActiveBrowser.PageTitle);

            //Check all data has default data.
            ValidateRecipeLocations();


        }



        [Test]
        public void TestSaveRecipeButton()
        {
            ActiveBrowser.NavigateTo(BASIC_SETUP_RECIPE_PAGE);
            Assert.AreEqual(BASIC_SETUP_RECIPE_PAGE_TITLE, ActiveBrowser.PageTitle, "Actual page title = \"{0}\"", ActiveBrowser.PageTitle);

            //CMP WIP This needs to be resolved how to change data in Radgrid cells
            ChangeLocationDetails();


            RadButton button = Find.ById<RadButton>(SAVE_BUTTON);
            Assert.AreEqual(SAVE_BUTTON, button.ClientId);
            button.Click();

            System.Threading.Thread.Sleep(2000);

            //Check data filled in behind; remember that the Server version is held by IIS. Can onlycheck against this test applications's local data copy.
            ValidateRecipeLocations();

        }





        private void ValidateRecipeLocations()
        {
            RadGrid bumpGrid = Find.ById<RadGrid>(BUMP_LOCATION_GRID);
            GridTableView masterTable = bumpGrid.MasterTable;

            Assert.AreEqual(s_locationMessages.Count, masterTable.DataItems.Count);

            for (int i = 0; i < s_locationMessages.Count; i++)
            {
                GUIBasicRecipeLocationMessage rlm = s_locationMessages[i];
                Assert.AreEqual(rlm.GUIIndex.ToString(), masterTable.DataItems[i].DataCells[BUMP_LOCATION_GUI_INDEX_CELL].CellText);
                Assert.AreEqual("display: none;", masterTable.DataItems[i].DataCells[BUMP_LOCATION_GUI_INDEX_CELL].Attributes[0].Value);
                Assert.AreEqual(rlm.x_coord.ToString(), masterTable.DataItems[i].DataCells[BUMP_LOCATION_XPOSITION_CELL].CellText, "Row=" + i.ToString() + ", Cell=XPosition");
                Assert.AreEqual(rlm.y_coord.ToString(), masterTable.DataItems[i].DataCells[BUMP_LOCATION_YPOSITION_CELL].CellText, "Row=" + i.ToString() + ", Cell=YPosition");
            }
        }


        /// <summary>
        /// This test is flaky!!!
        /// It has worked but I cannot determine the circumstances that will make it consistently work.
        /// </summary>
        private void ChangeLocationDetails()
        {
            string logIdentifier = "TestBasicSetupRecipe:ChangeLocationDetails:";

            for (int i = 0; i < s_locationMessages.Count; i++)
            {
                s_locationMessages[i].x_coord = (i + SEED_START + SEED_INCREMENT);
                s_locationMessages[i].y_coord = (i + SEED_START);
            }

            RadGrid bumpGrid = Find.ById<RadGrid>(BUMP_LOCATION_GRID);
            GridTableView masterTable = bumpGrid.MasterTable;

            int[] cellList = { BUMP_LOCATION_YPOSITION_CELL, BUMP_LOCATION_XPOSITION_CELL };
            for (int i = 0; i < masterTable.DataItems.Count; i++)
            {
                float seed = SEED_START;
                HtmlTableRow rowToEdit = bumpGrid.MasterTable.Rows[i];

                foreach (int j in cellList)
                {
                    System.Console.WriteLine(logIdentifier + "CellToEdit: Row(" + i.ToString() + "," + j.ToString() + ") ");
                    // Locate the cell to be updated
                    HtmlTableCell cellToEdit = bumpGrid.MasterTable.Rows[i].Cells[j];
                    // Put the cell in edit mode by a mouse click
                    try
                    {
                        cellToEdit.Click();
                        System.Threading.Thread.Sleep(200);
                        cellToEdit.Refresh();
                    }
                    catch (Exception ex)
                    {
                        System.Console.WriteLine(logIdentifier + "CellToEdit Exception Row(" + i.ToString() + "," + j.ToString() + ") ");
                        cellToEdit.Click();
                        System.Threading.Thread.Sleep(200);
                        cellToEdit.Refresh();

                    }
                    Manager.ActiveBrowser.RefreshDomTree();
                    System.Threading.Thread.Sleep(200);

                    // Locate the <input> element that is added when the cell enters edit mode
                    HtmlInputText inputBox = cellToEdit.Find.ByTagIndex<HtmlInputText>("input", 0);
                    int waitTime = 2000;
                    for (int delay = 0; ; delay += 100)
                    {

                        if (delay >= waitTime) Assert.Fail(logIdentifier + "Row(" + i.ToString() + "," + j.ToString() + ") :  Timeout.  ");
                        try
                        {

                            if ((inputBox = cellToEdit.Find.ByTagIndex<HtmlInputText>("input", 0)) != null) break;
                        }
                        catch (Exception ex)
                        {
                            System.Console.WriteLine(logIdentifier + ":  Exception ", ex);
                        }
                        System.Threading.Thread.Sleep(100);
                    }

                    inputBox.Click();
                    inputBox.SetValue("innerText", (i + seed).ToString());
                    inputBox.Text = (i + seed).ToString();

                    // Give it a new value
                    //Actions.SetText(inputBox, (i + seed).ToString());
                    System.Console.WriteLine(logIdentifier + "CellToEdit: Row(" + i.ToString() + "," + j.ToString() + ") InnerText" + inputBox.Value);

                    seed += SEED_INCREMENT;
                    System.Threading.Thread.Sleep(500);
                }


            }

        }

    }
}
