﻿using System;
using System.Globalization;
using System.IO;
using System.Linq;
using System.Windows.Forms;


using ArtOfTest.WebAii.Core;
using ArtOfTest.WebAii.ObjectModel;
using ArtOfTest.WebAii.TestTemplates;
using ArtOfTest.WebAii.TestAttributes;
using Telerik.WebAii.Controls.Html;

using NUnit.Framework;
using Core = NUnit.Core;

using System.Diagnostics;


namespace TelerikWebApp2.Tests
{
 
    [TestFixture]
    public class WebTestBase : BaseTest
    {

        #region WEB Constants
        protected const int    WAIT_FOR_ELEMENT_TIMEOUT = 1000;
        protected const string WRAPPER = "_wrapper";

        protected const string m_SQLscriptFolder = @"..\..\..\SQLscripts\";
        protected const string m_WebPageFolder = @"..\..\..\..\..\RuntimeGUI\RuntimeGUI";

        protected const string BASIC_SETUP_RECIPE_PAGE = @"~/Default.aspx";
         
        protected const string BASIC_SETUP_RECIPE_PAGE_TITLE = @"XM8000 - Setup";
        #endregion WEB Constants

        private Process m_WebServerProcess = null;
        private string m_PathWebServer = @"C:\Program Files\IIS Express\iisexpress.exe";
        private string m_Port = "50535";
        protected string m_RootUrl = string.Empty;


        private string m_PagesDir = string.Empty;

 

        #region WebAii attributes
        protected string PAGE_NAME = "~/Default.aspx";

        internal Browser m_CMSBrowser = null;
        internal Browser m_JobsBrowser = null;
        internal Browser m_TrackingBrowser = null;
        internal Browser m_ManualBrowser = null;
        #endregion WebAii

  
 
        /// <summary>
        /// Initialization for each test.
        /// </summary>
        [TestFixtureSetUp]
        public void WebBaseTestSetupFixture()
        {
            System.Console.WriteLine("********** TestFixtureSetUp: WebBaseTestSetupFixture  *********");

            #region WebAii Initialization

            // Initializes WebAii manager to be used by the test case.
            // If a WebAii configuration section exists, settings will be
            // loaded from it. Otherwise, will create a default settings
            // object with system defaults.
            //
            // Note: We are passing in a delegate to the NUnit's TestContext.Out.
            // WriteLine() method. This way any logging
            // done from WebAii (i.e. Manager.Log.WriteLine()) is
            // automatically logged to same output as NUnit.
            //
            // If you do not care about unifying the log, then you can simply
            // initialize the test by calling Initialize() with no parameters;
            // that will cause the log location to be picked up from the config
            // file if it exists or will use the default system settings.
            // You can also use Initialize(LogLocation) to set a specific log
            // location for this test.
            //Initialize(new TestContextWriteLine(Core.TestExecutionContext.CurrentContext.Out.WriteLine));

            // If you need to override any other settings coming from the
            // config section or you don't have a config section, you can
            // comment the 'Initialize' line above and instead use the
            // following:

            // This will get a new Settings object. If a configuration
            // section exists, then settings from that section will be
            // loaded

            // Start IIS before this
            string LocalHostUrl = string.Format("http://localhost:{0}", m_Port);
            string VirtualPath = "/";
            m_RootUrl = LocalHostUrl + VirtualPath;
            Settings settings = GetSettings();
            settings.Web.BaseUrl = m_RootUrl;
            //CMP Try IIS outside of test
            //settings.Web.LocalWebServer = LocalWebServerType.None;
            settings.ClientReadyTimeout = 40000;    //double default
            //CMP Set these to see on Page what has been seleetced
            //settings.AnnotateExecution = true;
            //settings.AnnotationMode = AnnotationMode.All;

            //settings.LogLocation = System.IO.Path.GetFullPath(Path.Combine(Core.TestExecutionContext.CurrentContext.CurrentDirectory, m_LogFilePath));
            // Now call Initialize again with your updated settings object
            Initialize(settings, new TestContextWriteLine(Core.TestExecutionContext.CurrentContext.Out.WriteLine));

            #endregion
            //
            // Place any additional initialization here
            //

            InitWebTest();

            //Start THE browswer for the test
            Manager.LaunchNewBrowser();
            //Set window onto 2nd screen half width
            ActiveBrowser.Window.Move(new System.Drawing.Rectangle(1920, 0, 960, 1200), true);

            SetupFixture();

        }

        public virtual void SetupFixture()
        {
            //Use this for local Fixture startup actions
        }

        [SetUp]
        public virtual void  Setup()
        {
            //Use this for local startup actions
            System.Console.WriteLine("********** Running test: " + TestContext.CurrentContext.Test.FullName + " *********");
        }


         
         [TestFixtureTearDown]
         public void WebBaseTestTeardownFixture()
         {
             System.Console.WriteLine("********** TestFixtureTearDown: WebBaseTestTeardownFixture  *********");
             TeardownFixture();

             for (int b = 0; b < Manager.Browsers.Count; b++ )
             {
                 Manager.Browsers[b].Close();
             }
             m_CMSBrowser = null;
             m_JobsBrowser = null;
             m_TrackingBrowser = null;
             m_ManualBrowser = null;

             this.CleanUp();
             
             ShutdownIIS();
 
         }

         public virtual void TeardownFixture()
         {
             //Use this for local Teardown actions
         }


         /// <summary>
         /// Clean up after each test.
         /// </summary>
         [TearDown]
         public virtual void Teardown()
         {
             string resultStr = "********** Finished test: " + TestContext.CurrentContext.Result.Status + ": " + TestContext.CurrentContext.Test.FullName + " *********";
             switch (TestContext.CurrentContext.Result.Status)
             {
                 case TestStatus.Inconclusive:
                     System.Console.WriteLine(resultStr);
                     break;
                 case TestStatus.Failed:
                     System.Console.WriteLine(resultStr);
                     break;
                 case TestStatus.Passed:
                 case TestStatus.Skipped:
                 default:
                     System.Console.WriteLine(resultStr);
                     break;
             }
         }


 
        #region HelperMethods
        protected void ValidateNumericText(string numericId, string validatorId, bool withLogging = false)
        {
            RadNumericTextBox numericText = Find.ById<RadNumericTextBox>(numericId + WRAPPER);
            numericText.Focus();
            numericText.MouseClick();
            numericText.InputValue = "";
            Manager.Desktop.KeyBoard.KeyPress(Keys.Tab, 100, 2);    //Loose focus

            Element validator = Find.ById(validatorId);
            validator.Wait.ForExists(WAIT_FOR_ELEMENT_TIMEOUT);
            if (withLogging) System.Console.WriteLine("WebTestBase:ValidateNumericText:" + validatorId + ":[0]: " + validator.Attributes[0].Value);
            if (withLogging) System.Console.WriteLine("WebTestBase:ValidateNumericText:" + validatorId + ":[1]: " + validator.Attributes[1].Value);
            if (withLogging) System.Console.WriteLine("WebTestBase:ValidateNumericText:" + validatorId + ":InnerText: " + validator.InnerText);
            Assert.AreNotEqual("color: red; display: none;", validator.Attributes[1].Value, validatorId);
            Assert.AreEqual("color: red; display: inline;", validator.Attributes[1].Value, validatorId);

        }

        protected void ValidateText(string textId, string validatorId, bool withLogging = false)
        {
            RadTextBox textBox = Find.ById<RadTextBox>(textId + WRAPPER);
            textBox.Focus();
            textBox.MouseClick();
            textBox.InputValue = "";
            Manager.Desktop.KeyBoard.KeyPress(Keys.Tab, 100, 2);    //Loose focus

            Element validator = Find.ById(validatorId);
            validator.Wait.ForExists(WAIT_FOR_ELEMENT_TIMEOUT);
            if (withLogging) System.Console.WriteLine("WebTestBase:ValidateText:" + validatorId + ":[0]: " + validator.Attributes[0].Value);
            if (withLogging) System.Console.WriteLine("WebTestBase:ValidateText:" + validatorId + ":[1]: " + validator.Attributes[1].Value);
            if (withLogging) System.Console.WriteLine("WebTestBase:ValidateText:" + validatorId + ":InnerText: " + validator.InnerText);
            Assert.AreNotEqual("color: red; display: none;", validator.Attributes[1].Value, validatorId);
            Assert.AreEqual("color: red; display: inline;", validator.Attributes[1].Value, validatorId);

        }

        protected void ValidateTextContent(string textId, string validatorId, string invalidText, bool withLogging = false)
        {
            RadTextBox textBox = Find.ById<RadTextBox>(textId + WRAPPER);
            textBox.Focus();
            textBox.MouseClick();
            textBox.InputValue = invalidText;
            Manager.Desktop.KeyBoard.KeyPress(Keys.Tab, 100, 2);    //Loose focus

            Element validator = Find.ById(validatorId);
            validator.Wait.ForExists(WAIT_FOR_ELEMENT_TIMEOUT);
            if (withLogging) System.Console.WriteLine("WebTestBase:ValidateText:" + validatorId + ":[0]: " + validator.Attributes[0].Value);
            if (withLogging) System.Console.WriteLine("WebTestBase:ValidateText:" + validatorId + ":[1]: " + validator.Attributes[1].Value);
            if (withLogging) System.Console.WriteLine("WebTestBase:ValidateText:" + validatorId + ":InnerText: " + validator.InnerText);
            Assert.AreNotEqual("color: red; display: none;", validator.Attributes[1].Value, validatorId);
            Assert.AreEqual("color: red; display: inline;", validator.Attributes[1].Value, validatorId);

        }

        protected void ValidateRangeNumericText(string numericId, string rangeValidatorId, int minValue, int maxValue, bool withLogging = false)
        {
            RadNumericTextBox numericText = Find.ById<RadNumericTextBox>(numericId + WRAPPER);
            numericText.Focus();
            numericText.MouseClick();
            numericText.InputValue = "";
            Manager.Desktop.KeyBoard.KeyPress(Keys.Tab, 100, 2);    //Loose focus

            //Less than Min... as default values then check set to min
            numericText.Focus();
            numericText.MouseClick();
            numericText.InputValue = (minValue - 1).ToString();
            Manager.Desktop.KeyBoard.KeyPress(Keys.Tab, 100, 2);    //Loose focus
            Element rangeValidator = Find.ById(rangeValidatorId);
            rangeValidator.Wait.ForExists(WAIT_FOR_ELEMENT_TIMEOUT);
            if (withLogging) System.Console.WriteLine("WebTestBase:ValidateRangeNumericText:" + rangeValidatorId + ":[0]: " + rangeValidator.Attributes[0].Value);
            if (withLogging) System.Console.WriteLine("WebTestBase:ValidateRangeNumericText:" + rangeValidatorId + ":[1]: " + rangeValidator.Attributes[1].Value);
            if (withLogging) System.Console.WriteLine("WebTestBase:ValidateRangeNumericText:" + rangeValidatorId + ":InnerText: " + rangeValidator.InnerText + ", Min=" + minValue.ToString() + ", Max=" + maxValue.ToString());
            Assert.AreEqual("color: red; display: none;", rangeValidator.Attributes[1].Value, rangeValidatorId);  //validator not shown
            Assert.AreEqual(numericText.MinValue.ToString(), numericText.InputValue, rangeValidatorId);

            //More than Max ... as default values then check set to max
            numericText.Focus();
            numericText.MouseClick();
            numericText.InputValue = (maxValue + 1).ToString();
            Manager.Desktop.KeyBoard.KeyPress(Keys.Tab, 100, 2);    //Loose focus
            rangeValidator = Find.ById(rangeValidatorId);
            rangeValidator.Wait.ForExists(WAIT_FOR_ELEMENT_TIMEOUT);
            Assert.AreEqual("color: red; display: none;", rangeValidator.Attributes[1].Value);  //Validator not shown
            Assert.AreEqual(numericText.MaxValue.ToString(), numericText.InputValue);

        }

        protected void ValidateNumericText2Validators(string numericId, string validatorId, string rangeValidatorId, double minValue, double maxValue, bool withLogging = false)
        {
            RadNumericTextBox numericText = Find.ById<RadNumericTextBox>(numericId + WRAPPER);
            numericText.Focus();
            numericText.MouseClick();
            numericText.InputValue = "";
            Manager.Desktop.KeyBoard.KeyPress(Keys.Tab, 100, 2);    //Loose focus

            Element validator = Find.ById(validatorId);
            validator.Wait.ForExists(WAIT_FOR_ELEMENT_TIMEOUT);
            Assert.AreEqual("color: red; display: inline;", validator.Attributes[1].Value, validatorId);  

            //Less than Min... as default values then check set to min
            numericText.Focus();
            numericText.MouseClick();
            numericText.InputValue = (minValue - 1).ToString();
            Manager.Desktop.KeyBoard.KeyPress(Keys.Tab, 100, 2);    //Loose focus
            Element rangeValidator = Find.ById(rangeValidatorId);
            rangeValidator.Wait.ForExists(WAIT_FOR_ELEMENT_TIMEOUT);
            if (withLogging) System.Console.WriteLine("WebTestBase:ValidateRangeNumericText:" + rangeValidatorId + ":[0]: " + rangeValidator.Attributes[0].Value);
            if (withLogging) System.Console.WriteLine("WebTestBase:ValidateRangeNumericText:" + rangeValidatorId + ":[1]: " + rangeValidator.Attributes[1].Value);
            if (withLogging) System.Console.WriteLine("WebTestBase:ValidateRangeNumericText:" + rangeValidatorId + ":InnerText: " + rangeValidator.InnerText + ", Min=" + minValue.ToString() + ", Max=" + maxValue.ToString());
            Assert.AreEqual("color: red; display: inline;", rangeValidator.Attributes[1].Value, rangeValidatorId);  

            //More than Max ... as default values then check set to max
            numericText.Focus();
            numericText.MouseClick();
            numericText.InputValue = (maxValue + 1).ToString();
            Manager.Desktop.KeyBoard.KeyPress(Keys.Tab, 100, 2);    //Loose focus
            rangeValidator = Find.ById(rangeValidatorId);
            rangeValidator.Wait.ForExists(WAIT_FOR_ELEMENT_TIMEOUT);
            Assert.AreEqual("color: red; display: inline;", rangeValidator.Attributes[1].Value);  

        }  

        /// <summary>
        /// Allow the WebPage to be viewd at the end of a test
        /// </summary>
        /// <param name="duration"></param>
        public void Pause(int duration=3000)
        {
            System.Threading.Thread.Sleep(duration);
        }

         public void WaitForGridCellData(GridTableView masterTable, int itemIndex, int cellIndex, string expectedData, int waitTime = 2000)
        {
            string logIdentifier = "WebTestBase: WaitForGridCellData: " + masterTable.ID + ".DataItems[" + itemIndex.ToString()
                                   + "].DataCells[" + cellIndex.ToString() + "] ExpectedData=" + expectedData;
            for (int delay = 0; ; delay += 100)
            {
                //FOR DIAGNOSTIC System.Console.WriteLine(logIdentifier + ":  Delay "+ delay.ToString());

                if (delay >= waitTime) Assert.Fail(logIdentifier + " :  Timeout. Actual data " + masterTable.DataItems[itemIndex].DataCells[cellIndex].CellText);
                try
                {
                    if (masterTable.DataItems[itemIndex].DataCells[cellIndex].CellText == expectedData) break;
                }
                catch (Exception ex)
                {
                    System.Console.WriteLine(logIdentifier + ":  Exception ", ex);
                }
                System.Threading.Thread.Sleep(100);
            }
        }

        public void WaitForValue(Browser browser, string elementName,  string expectedData, int waitTime = 2000)
        {
            Element element;

            string logIdentifier = "WebTestBase: WaitForValue: " + elementName + ": ExpectedData=" + expectedData;
            for (int delay = 0; ; delay += 100)
            {
                //FOR DIAGNOSTIC System.Console.WriteLine(logIdentifier + ":  Delay "+ delay.ToString());
                element = browser.Find.ByName(elementName);
                if (delay >= waitTime) Assert.Fail(logIdentifier + " :  Timeout. Actual data " + element.GetValue("value", ""));
                try
                {
                    if (element.GetValue("value", "") == expectedData) break;
                }
                catch (Exception ex)
                {
                    //Item may not be there yet
                    System.Console.WriteLine(logIdentifier + " delay=" + delay.ToString() + ":  Exception ", ex);
                }
                System.Threading.Thread.Sleep(100);
            }
        }

        protected void DumpFooterLog(Browser browser)
        {
            try
            {
                RadTextBox footerLog = browser.Find.ById<RadTextBox>("ctl00_FooterContent_FooterText_wrapper");
                if (footerLog != null)
                {
                    string footerStr = Environment.NewLine + "FOOTER LOG (" + browser.PageTitle + ") ==============================" + Environment.NewLine;
                    string tempstr = footerLog.TextBoxValue;
                    footerStr += tempstr.Replace("<br>", "\n");
                    footerStr += Environment.NewLine + "END FOOTER LOG (" + browser.PageTitle + ") ==============================";
                    System.Console.WriteLine("WebTestBase: DumpFooterLog: Footer Log for " + browser.PageTitle + footerStr);
                }
            }
            catch
            {

            }
        }

        protected void ClearFooterLog()
        {
            System.Console.WriteLine("WebTestBase: ClearFooterLog: CLEARED" );
            Element FooterEl = ActiveBrowser.Find.ById("ctl00_FooterContent_FooterText");
            ActiveBrowser.Actions.SetText(FooterEl, "");
        }

        public static string CapitaliseFirst(string name)
        {
            return String.Format("{0}{1}", name.First().ToString().ToUpperInvariant(), name.Substring(1).ToLowerInvariant());
        }

        #endregion HelperMethods

        #region Private Test Fixture Methods
        private void InitWebTest()
        {

            bool IsIISStarted = false;
            Process[] prs = Process.GetProcesses();

            foreach (Process pr in prs)
            {
                if (pr.ProcessName.Contains("iisexpress"))
                {
                    IsIISStarted = true;
                }
            }
            if (!IsIISStarted)
            {
                StartIISExpress();
            }

        }

        private void ShutdownIIS()
        {
            //IF IIS started by test ... then shutdown by test
            if (m_WebServerProcess != null)
            {
                m_WebServerProcess.Kill();
                m_WebServerProcess.Dispose();
                m_WebServerProcess = null;


                Process[] prs = Process.GetProcesses();

                foreach (Process pr in prs)
                {
                    if (pr.ProcessName.Contains("iisexpress"))
                    {
                        pr.Kill();
                    }
                }
            }
        }

        private void StartIISExpress()
        {

            //Start IIS with the RuntimeGUI
            var index = Environment.CurrentDirectory.LastIndexOf("\\TelerikWebApp2\\");
            string projectDir = Environment.CurrentDirectory.Remove(index);
            string arguments = "/config:"
                             + '"' + projectDir + '/' + @"TelerikWebApp2/applicationhost.config" + '"'
                             + " /site:TelerikWebApp2";


            System.Console.WriteLine("BaseDBConnection: StartIISExpress:  " + m_PathWebServer + " " + arguments);

            // create a new process to start the ASP.NET Development Server
            m_WebServerProcess = new Process();

            /// configure the web server
            m_WebServerProcess.StartInfo.FileName = m_PathWebServer;
            m_WebServerProcess.StartInfo.Arguments = arguments;
            m_WebServerProcess.StartInfo.CreateNoWindow = false;
            m_WebServerProcess.StartInfo.UseShellExecute = false;
            m_WebServerProcess.StartInfo.RedirectStandardOutput = false;    //Do NOT redirect as this stops the IIS working correctly
            m_WebServerProcess.StartInfo.RedirectStandardInput = false;
            m_WebServerProcess.StartInfo.RedirectStandardError = true;
            m_WebServerProcess.EnableRaisingEvents = true;
            m_WebServerProcess.Exited += IISServer_Exited;

            // start the web server
            bool result = m_WebServerProcess.Start();
            if (!result) System.Console.WriteLine("WebTestBase: StartIISExpress:  Failed.");
        }

        private void IISServer_Exited(object sender, EventArgs e)
        {
            var index = Environment.CurrentDirectory.LastIndexOf("\\bin\\");
            string projectDir = Environment.CurrentDirectory.Remove(index);

            System.Console.WriteLine("WebTestBase: IISServer_Exited:");
            try
            {
                var process = (Process)sender;
                if (m_WebServerProcess.StartInfo.RedirectStandardError)
                {
                    using (var f = File.CreateText(projectDir + @"IISerr" + "TestRadControlsWebApp1.log"))
                    {
                        f.WriteLine(process.StandardError.ReadToEnd());
                    }
                }
                if (m_WebServerProcess.StartInfo.RedirectStandardOutput)
                {
                    using (var f = File.CreateText(projectDir + @"IISout" + "TestRadControlsWebApp1.log"))
                    {
                        f.WriteLine(process.StandardOutput.ReadToEnd());
                    }
                }

            }
            catch (SystemException ex)
            {
                System.Console.WriteLine("WebTestBase: IISServer_Exited:  Exception ", ex);

            }
        }

        #endregion Private Test Fixture Methods

    }
}
